#include "IC555.h"

extern PegBitmap gbastableBitmap;
extern PegBitmap gbmonoestableBitmap;

IC555::IC555(PegRect rect, CPMainFrame* frame):CPModuleWindow(rect,0,0,frame)
{
init();
}

int IC555::init()
{   
   	const WORD wStyle = FF_NONE|TJ_LEFT|AF_TRANSPARENT| TT_COPY;
	const SIGNED margin = 10;
	const SIGNED width=mClient.Width()-2*margin;
    
   	PEGCHAR* chrmu = "\xED\x72"; 

    PegRect rectangle;
 
 	lin1 = new PegPrompt(margin, 75, width, "DC(%):",0,wStyle);
	in1 = new CPPegString(margin+40,75,width - 70,"",0, FF_THIN|AF_ENABLED|EF_EDIT);
	AddR(lin1);
	AddR(in1);

 	lin2 = new PegPrompt(margin, 90, width, "f(Hz):",0,wStyle);
	in2 = new CPPegString(margin+40,90,width - 70,"",0, FF_THIN|AF_ENABLED|EF_EDIT);
	AddR(lin2);
	AddR(in2);
     	
    lin3 = new PegPrompt(margin, 105, width, "C("+CPString(chrmu)+"F):",0,wStyle);
	in3 = new CPPegString(margin+40,105,width - 70,"",0,FF_THIN|AF_ENABLED|EF_EDIT);
	AddR(lin3);
	AddR(in3);
    	
     //rectangle.Set(MAINFRAME_LEFT, MAINFRAME_TOP, MAINFRAME_RIGHT, MAINFRAME_BOTTOM);
    rectangle.Set(25, 125, 90, 140);
   	PegTextButton* b3 = new PegTextButton(rectangle, "Calcular!", BOTO, AF_ENABLED|TJ_CENTER);
	AddR(b3);
	
    rectangle.Set(MAINFRAME_LEFT, MAINFRAME_TOP, MAINFRAME_RIGHT, MAINFRAME_TOP+75);
 	imatge = new PegBitmapButton(rectangle,&gbastableBitmap,0,AF_TRANSPARENT,TYPE_PROMPT,FALSE);
 	AddR(imatge); 
 	
    out1 = new PegPrompt(margin, 150, width, "",0,wStyle);
	AddR(out1);
    out2 = new PegPrompt(margin, 165, width, "",0,wStyle);
	AddR(out2);
    out3 = new PegPrompt(margin, 180, width, "",0,wStyle);
	AddR(out3);
 
    astable=true;
    SetDefaultFocus(in1);
 
	return 0;
}

void IC555::AddUI()
{
// create text buttons for the toolbar
PegTextButton* b1 = new PegTextButton(1,1, "ASTABLE", MODEAST, AF_ENABLED|TT_COPY);
PegTextButton* b2 = new PegTextButton(2,1, "MONOESTABLE", MODEMONO, AF_ENABLED|TT_COPY);

//position and enable the buttons
m_ui->AddToolbarButton(b1);
m_ui->AddToolbarButton(b2);
}

void IC555::Draw()
{
	BeginDraw();
	Invalidate(mClient);
	DrawFrame();
	DrawChildren();
	EndDraw();
}

SIGNED IC555::Message(const PegMessage &Mesg)
{
   	PEGCHAR* chrmu = "\xED\x72"; 
    WORD estil_visible = FF_THIN|AF_ENABLED|EF_EDIT;
    WORD estil_invisible = FF_NONE|AF_TRANSPARENT|EF_EDIT;
    
    
	switch(Mesg.wType)
	{
		case SIGNAL(BOTO, PSF_CLICKED):
		{
             Calcula();
		}
		break;
		case SIGNAL(MODEAST, PSF_CLICKED):
		{
             imatge->SetBitmap(&gbastableBitmap);
             lin1->DataSet("DC(%):");
             lin2->DataSet("f(Hz):");
             lin3->DataSet("C("+CPString(chrmu)+"F):");                          
             in1->DataSet("");
             in2->DataSet("");
             in3->DataSet("");
             in3->FrameStyle(estil_visible);
             out1->DataSet("");
             out2->DataSet("");
             out3->DataSet("");
             Draw();
             astable=true;
		}
		break;
		case SIGNAL(MODEMONO, PSF_CLICKED):
		{
             imatge->SetBitmap(&gbmonoestableBitmap);
             lin1->DataSet("t(s):");
             lin2->DataSet("C("+CPString(chrmu)+"F):");
             lin3->DataSet("");
             in1->DataSet("");
             in2->DataSet("");
             in3->DataSet("");
             in3->FrameStyle(estil_invisible);
             out1->DataSet("");
             out2->DataSet("");
             out3->DataSet("");            
             Draw();
             astable=false;
		}
		break;
		
		default:
		return CPModuleWindow::Message(Mesg);
	}
	return 0;
}

WORD IC555::Dialog(char* title, char* message)
{
	PegRect rr = GetMainFrame()->TopAppRectangle();
	rr.wBottom -= 30;
	rr.wRight -= 10;
	rr.wTop += 10;
	rr.wLeft += 10;
	PegMessageWindow *msg = new PegMessageWindow(rr, title, message, MW_OK, 0,0,0);
	return msg->Execute();
}

void IC555::Calcula()
{
     OBCD aux, aux2, DC, T, Rb;
     PEGCHAR* chrohm = "\xEC\x7E"; 
   	 //PEGCHAR* chrmu = "\xED\x72"; 
   	 
   	 if(astable)
   	 {
   	     /* CALCULA ASTABLE VALS */
   	     /* in1: DC
   	        in2: f(Hz)
   	        in3: C(uF)
   	        */
         // Calcula Toff
         aux = CPExpression(in1->DataGet()); // aux = DC(%)
         ExecuteToOBCD("100",aux2);
         Cal_div_OBC(&aux,&aux2); // aux = DC/100
         DC = aux;
         Cal_setn_OBC(1,&aux2);
         Cal_sub_OBC(&aux2,&aux); // aux2 = 1-DC/100
         aux = CPExpression(in2->DataGet());
         Cal_inverse_OBC(&aux); // aux = T
         T = aux;
         Cal_mul_OBC(&aux,&aux2); // aux = Toff
         
         // Calcula Rb
         aux2 = CPExpression(in3->DataGet());
         Cal_div_OBC(&aux,&aux2);
         ExecuteToOBCD("0.000000693",aux2);
         Cal_div_OBC(&aux,&aux2);
         out2->DataSet("Rb = "+CPString(aux)+" "+CPString(chrohm));
         Rb = aux;
         
         // Calcula Ton
         aux = T;
         Cal_mul_OBC(&aux,&DC); // aux = Ton
         
         // Calcula Ra
         Cal_div_OBC(&aux,&aux2);
         aux2 = CPExpression(in3->DataGet());
         Cal_div_OBC(&aux,&aux2);
         Cal_sub_OBC(&aux,&Rb);
         out1->DataSet("Ra = "+CPString(aux)+" "+CPString(chrohm));
   	 }
   	 else
   	 {
   	     /* CALCULA MONOESTABLE VALS */
     	     /* in1: t(s)
   	        in2: C(uF)
   	        */
 	     aux = CPExpression(in1->DataGet());
 	     ExecuteToOBCD("0.0000011",aux2);
 	     Cal_div_OBC(&aux,&aux2);
 	     aux2 = CPExpression(in2->DataGet());
 	     Cal_div_OBC(&aux,&aux2);
 	     out1->DataSet("Ra = "+CPString(aux)+" "+CPString(chrohm));
 	     out2->DataSet("");
   	 } 
           
     Draw();
}   

extern "C" char *ExtensionGetLang(ID_MESSAGE MessageNo)
{
return "";
}
